==> audiochain-1.1.5-src.tgz: org/audiochain/ui/gui/mixer/AudioProjectComponent.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.gui.mixer;

import static javax.swing.ScrollPaneConstants.*;
import static org.audiochain.ui.gui.CommonComponentFactory.*;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ContainerAdapter;
import java.awt.event.ContainerEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.audiochain.model.AudioDevice;
import org.audiochain.model.AudioDeviceChain;
import org.audiochain.model.AudioDeviceChainAdapter;
import org.audiochain.model.AudioProject;
import org.audiochain.model.AudioProjectAdapter;
import org.audiochain.model.AudioTrack;
import org.audiochain.ui.gui.Dialog;
import org.audiochain.ui.gui.ImageButtonComponent;


public class AudioProjectComponent extends JPanel {
	private static final long serialVersionUID = 1L;
	private final AudioProject audioProject;
	private JComponent trackContainer;
	private Component glue;
	private PerformanceControlComponent performanceControlComponent;
	private Map<AudioTrack, AudioTrackComponent> audioTrackComponentMap;

	public AudioProjectComponent(AudioProject audioProject) {
		if (audioProject == null) {
			throw new IllegalArgumentException("Project must not be null.");
		}
		this.audioProject = audioProject;
		setName(audioProject.getName());
		init();
	}

	private void init() {
		setOpaque(false);
		setLayout(new BorderLayout());
		performanceControlComponent = new PerformanceControlComponent(audioProject);

		trackContainer = Box.createHorizontalBox();
		trackContainer.addContainerListener(new ContainerAdapter() {
			@Override
			public void componentRemoved(ContainerEvent e) {
				trackContainer.revalidate();
				trackContainer.repaint();
			}
		});
		glue = Box.createHorizontalGlue();
		trackContainer.setOpaque(false);
		trackContainer.add(glue);
		add(createScrollPane(trackContainer), BorderLayout.CENTER);
		Collection<AudioTrack> tracks = audioProject.getAudioTracks();
		for (AudioTrack audioTrack : tracks) {
			addAudioTrackComponent(audioTrack);
		}
		updateTrackBox();
		createMasterBox();

		add(performanceControlComponent, BorderLayout.SOUTH);

		audioProject.addAudioProjectChangeListener(new AudioProjectAdapter() {
			@Override
			public void audioTrackAdded(AudioTrack track) {
				addAudioTrackComponent(track);
				updateTrackBox();
			}

			@Override
			public void audioTrackRemoved(AudioTrack track) {
				AudioTrackComponent removedComponent = getAudioTrackComponentMap().remove(track);
				trackContainer.remove(removedComponent);
				updateTrackBox();
			}

			@Override
			public void audioTrackMoved(AudioTrack track) {
				updateTrackBox();
			}
		});
		
		audioProject.getMasterAudioDeviceChain().addAudioDeviceChainListener(new AudioDeviceChainAdapter() {
			@Override
			public void audioDeviceRemoved(AudioDevice audioDevice, AudioDeviceChain audioDeviceChain) {
				revalidate();
			}
			@Override
			public void audioDeviceAdded(AudioDevice audioDevice, AudioDeviceChain audioDeviceChain) {
				revalidate();
			}
		});
	}

	private void createMasterBox() {
		Box masterBox = Box.createVerticalBox();
		masterBox.setOpaque(false);
		JScrollPane scrollPane = createScrollPane(masterBox);
		scrollPane.setVerticalScrollBarPolicy(VERTICAL_SCROLLBAR_ALWAYS);
		scrollPane.setHorizontalScrollBarPolicy(HORIZONTAL_SCROLLBAR_ALWAYS);
		add(scrollPane, BorderLayout.EAST);

		Box masterToolbarBox = Box.createHorizontalBox();
		masterToolbarBox.add(createAddTrackButton());
		masterToolbarBox.add(Box.createHorizontalGlue());
		masterBox.add(masterToolbarBox);

		masterBox.add(new AudioDeviceChainComponent(audioProject, audioProject.getMasterAudioDeviceChain(), true));

		JLabel label = new JLabel("Master");
		label.setAlignmentX(CENTER_ALIGNMENT);
		masterBox.add(label);
	}

	private Component createAddTrackButton() {
		ImageButtonComponent addTrackButton = new ImageButtonComponent("org/audiochain/ui/gui/mixer/plus-item.png");
		addTrackButton.setToolTipText("Adds a new track to the project (key n).");
		addTrackButton.setReleasedBorder(BorderFactory.createEmptyBorder());
		addTrackButton.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent event) {
				addNewTrack(audioProject, AudioProjectComponent.this);
			}
		});
		return addTrackButton;
	}

	static void addNewTrack(AudioProject audioProject, Object caller) {
		AudioProject templateAudioProject;
		try {
			templateAudioProject = audioProject.getTemplateProject();
		} catch (Exception ex) {
			ex.printStackTrace();
			Dialog.message(caller, "Error while loading audio project template", ex);
			templateAudioProject = null;
		}
		try {
			audioProject.addNewAudioTrack(templateAudioProject);
		} catch (Exception ex) {
			ex.printStackTrace();
			Dialog.message(caller, "Error while adding a track", ex);
		}
	}

	private void updateTrackBox() {
		trackContainer.remove(glue);
		Map<AudioTrack, AudioTrackComponent> audioTrackComponentMap = getAudioTrackComponentMap();
		for (AudioTrackComponent comp : audioTrackComponentMap.values()) {
			trackContainer.remove(comp);
		}
		for (AudioTrack track : audioProject.getAudioTracks()) {
			AudioTrackComponent comp = getAudioTrackComponentMap().get(track);
			trackContainer.add(comp);
		}
		trackContainer.add(glue);
		trackContainer.revalidate();
		trackContainer.repaint();
	}

	private void addAudioTrackComponent(AudioTrack audioTrack) {
		AudioTrackComponent comp = new AudioTrackComponent(audioProject, audioTrack);
		getAudioTrackComponentMap().put(audioTrack, comp);
	}

	private Map<AudioTrack, AudioTrackComponent> getAudioTrackComponentMap() {
		if (audioTrackComponentMap == null) {
			audioTrackComponentMap = new HashMap<AudioTrack, AudioTrackComponent>();
		}
		return audioTrackComponentMap;
	}
}

==> audiochain-1.1.6-src.tgz: org/audiochain/ui/gui/mixer/AudioProjectComponent.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.gui.mixer;

import static javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS;
import static javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS;
import static org.audiochain.ui.gui.CommonComponentFactory.createScrollPane;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ContainerAdapter;
import java.awt.event.ContainerEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.audiochain.model.AudioDevice;
import org.audiochain.model.AudioDeviceChain;
import org.audiochain.model.AudioDeviceChainAdapter;
import org.audiochain.model.AudioProject;
import org.audiochain.model.AudioProjectAdapter;
import org.audiochain.model.AudioTrack;
import org.audiochain.ui.gui.Dialog;
import org.audiochain.ui.gui.ImageButtonComponent;


public class AudioProjectComponent extends JPanel {
	private static final long serialVersionUID = 1L;
	private final AudioProject audioProject;
	private JComponent trackContainer;
	private Component glue;
	private PerformanceControlComponent performanceControlComponent;
	private Map<AudioTrack, AudioTrackComponent> audioTrackComponentMap;

	public AudioProjectComponent(AudioProject audioProject) {
		if (audioProject == null) {
			throw new IllegalArgumentException("Project must not be null.");
		}
		this.audioProject = audioProject;
		setName(audioProject.getName());
		init();
	}

	private void init() {
		setOpaque(false);
		setLayout(new BorderLayout());
		performanceControlComponent = new PerformanceControlComponent(audioProject);

		trackContainer = Box.createHorizontalBox();
		trackContainer.addContainerListener(new ContainerAdapter() {
			@Override
			public void componentRemoved(ContainerEvent e) {
				trackContainer.revalidate();
				trackContainer.repaint();
			}
		});
		glue = Box.createHorizontalGlue();
		trackContainer.setOpaque(false);
		trackContainer.add(glue);
		add(createScrollPane(trackContainer), BorderLayout.CENTER);
		Collection<AudioTrack> tracks = audioProject.getAudioTracks();
		for (AudioTrack audioTrack : tracks) {
			addAudioTrackComponent(audioTrack);
		}
		updateTrackBox();
		createMasterBox();

		add(performanceControlComponent, BorderLayout.SOUTH);

		audioProject.addAudioProjectChangeListener(new AudioProjectAdapter() {
			@Override
			public void audioTrackAdded(AudioTrack track) {
				addAudioTrackComponent(track);
				updateTrackBox();
			}

			@Override
			public void audioTrackRemoved(AudioTrack track) {
				AudioTrackComponent removedComponent = getAudioTrackComponentMap().remove(track);
				trackContainer.remove(removedComponent);
				updateTrackBox();
			}

			@Override
			public void audioTrackMoved(AudioTrack track) {
				updateTrackBox();
			}
		});
		
		audioProject.getMasterAudioDeviceChain().addAudioDeviceChainListener(new AudioDeviceChainAdapter() {
			@Override
			public void audioDeviceRemoved(AudioDevice audioDevice, AudioDeviceChain audioDeviceChain) {
				revalidate();
			}
			@Override
			public void audioDeviceAdded(AudioDevice audioDevice, AudioDeviceChain audioDeviceChain) {
				revalidate();
			}
		});
	}

	private void createMasterBox() {
		Box masterBox = Box.createVerticalBox();
		masterBox.setOpaque(false);
		JScrollPane scrollPane = createScrollPane(masterBox);
		scrollPane.setVerticalScrollBarPolicy(VERTICAL_SCROLLBAR_ALWAYS);
		scrollPane.setHorizontalScrollBarPolicy(HORIZONTAL_SCROLLBAR_ALWAYS);
		add(scrollPane, BorderLayout.EAST);

		Box masterToolbarBox = Box.createHorizontalBox();
		masterToolbarBox.add(createAddTrackButton());
		masterToolbarBox.add(Box.createHorizontalGlue());
		masterBox.add(masterToolbarBox);

		masterBox.add(new AudioDeviceChainComponent(audioProject, audioProject.getMasterAudioDeviceChain(), true));

		JLabel label = new JLabel("Master");
		label.setAlignmentX(CENTER_ALIGNMENT);
		masterBox.add(label);
	}

	private Component createAddTrackButton() {
		ImageButtonComponent addTrackButton = new ImageButtonComponent("org/audiochain/ui/gui/mixer/plus-item.png");
		addTrackButton.setToolTipText("Adds a new track to the project (key n).");
		addTrackButton.setReleasedBorder(BorderFactory.createEmptyBorder());
		addTrackButton.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent event) {
				addNewTrack(audioProject, AudioProjectComponent.this);
			}
		});
		return addTrackButton;
	}

	static void addNewTrack(AudioProject audioProject, Object caller) {
		AudioProject templateAudioProject;
		try {
			templateAudioProject = audioProject.getTemplateProject();
		} catch (Exception ex) {
			ex.printStackTrace();
			Dialog.message(caller, "Error while loading audio project template", ex);
			templateAudioProject = null;
		}
		try {
			audioProject.addNewAudioTrack(templateAudioProject);
		} catch (Exception ex) {
			ex.printStackTrace();
			Dialog.message(caller, "Error while adding a track", ex);
		}
	}

	private void updateTrackBox() {
		trackContainer.remove(glue);
		Map<AudioTrack, AudioTrackComponent> audioTrackComponentMap = getAudioTrackComponentMap();
		for (AudioTrackComponent comp : audioTrackComponentMap.values()) {
			trackContainer.remove(comp);
		}
		for (AudioTrack track : audioProject.getAudioTracks()) {
			AudioTrackComponent comp = getAudioTrackComponentMap().get(track);
			trackContainer.add(comp);
		}
		trackContainer.add(glue);
		trackContainer.revalidate();
		trackContainer.repaint();
	}

	private void addAudioTrackComponent(AudioTrack audioTrack) {
		AudioTrackComponent comp = new AudioTrackComponent(audioProject, audioTrack);
		getAudioTrackComponentMap().put(audioTrack, comp);
	}

	private Map<AudioTrack, AudioTrackComponent> getAudioTrackComponentMap() {
		if (audioTrackComponentMap == null) {
			audioTrackComponentMap = new HashMap<AudioTrack, AudioTrackComponent>();
		}
		return audioTrackComponentMap;
	}
}
